#!/bin/bash
###############################################################################
# The purpose of this script is to cleanup directories and files which the user
# does not have access to. These directories and files can stop an update
# because they take up disk space which is needed by the update to finish.
#
# The following items are cleaned up:
#    - nullmailer - In version 5.0.2 the nullmailer directory can grow unchecked
#                   and take up disk space. This was controlled in 5.1.2 however
#                   customers which did not update can run into an issue with
#                   This directory taking up lots of disk space.
#    - Python     - In version 5.1.5 two version of python made it onto the
#                   SecureSync filesystem. Python is not required to be on the
#                   SecureSync and was removed in version 5.1.7 of the software
#                   and later.
#    - Perl       - Perl is required by the transfer engine however it doesn't
#                   need to be on the SecureSync at the time of the update where
#                   the update bundle is unpacked. Perl will be removed to
#                   create space and then replaced before it needs to be used by
#                   the transfer engine.
#    - Wtmp       - Remove /var/log/wtmp to ensure login history does not grow
#                   without limit.
#    - SQLite     - SQLite database contains tables which are not maintained
#                   have grown beyond their limits. To ensure the database is
#                   maintained at less than a maximum size we perform maintenance
#                   on all tables.
#
###############################################################################

NULL="/var/nullmailer/queue/"
PYTHON="/usr/lib/python*"
PERL="/usr/lib/perl*"
MOTD="/etc/motd"
WTMP="/var/log/wtmp"
DB="/srv/www2/app/webroot/lafayette"
NTPPATH="/srv/www2/app/Console/Command/Task/NtpTask.php"
REFMONPATH="/srv/www2/app/Console/Command/Task/RefMonTask.php"

# List of possible database tables to maintain
declare -a tables=("log_oscillators" "log_gps_statuses" "log_events"
                   "log_sys_mons" "log_eth_mons" "log_ntp_stats"
                   "log_ref_mon_statuses")

# Max number of database table entries
tbl_limit=12000

# Check for only the supported versions
# Assumes version numbers are #.#.# with no double digits or letters
if [[ -e $MOTD ]] ; then
    cat $MOTD | grep "Version 5[.].[.]."
    if [[ "$?" -ne "0" ]] ; then
        exit 1 # A 5.x.x version was not found
    fi
else
    # If /etc/motd does not exist this is a unrecoverable failure
    # return the unit to the factory
    exit 1
fi

# set, Bash allows filename patterns which match no files to expand to a null
# string
shopt -s nullglob
# set, Bash includes filenames beginning with a '.' in the results of filename
# expansion
shopt -s dotglob

# Check nullmailer
check=($NULL/*)
if [[ ${#check[*]} -ge 1 ]]; then
    # the nullmailer queue directory is not empty, clean it
    find $NULL -name "*" -type f -print | xargs /bin/rm 2>&1 > /dev/null
fi

# Check Python
check=($PYTHON)
if [[ ${#check[*]} -ge 1 ]]; then
    # Python directories exist, clean it
    /bin/rm -rf $PYTHON 2>&1 > /dev/null
fi

# Attempt removing Perl
check=($PERL)
if [[ ${#check[*]} -ge 1 ]]; then
    # PERL directories exist, clean it
    /bin/rm -rf $PERL 2>&1 > /dev/null
fi

# Attempt removing /var/log/wtmp
check=($WTMP)
if [[ ${#check[*]} -ge 1 ]]; then
    # login history wtmp file exists, clean it
    /bin/rm -rf $WTMP 2>&1 > /dev/null
fi

# Check if a database exists in this version of the software
check=($DB)
if [[ ${#check[*]} -ge 1 ]]; then
    # Database exists lets loop through the possible tables
    # maintaining them if they exist and they have more than
    # tbl_limit entries.
    for i in "${tables[@]}"
    do
        # If the table is found in the database run through the maintenence process
        if [ "$(/usr/bin/sqlite3 "$DB" .tables | grep -cim 1 "$i")" -eq 1 ]; then
            count=$(/usr/bin/sqlite3 "$DB" "SELECT count(*) from "$i";")
            if [ $count -gt $tbl_limit ]; then
                # Maintain the table to only have the latest 10080 entries
                /usr/bin/sqlite3 $DB "pragma busy_timeout=1000; CREATE TABLE temp AS SELECT * FROM "$i" ORDER BY id DESC LIMIT 10080;
                                      DELETE FROM "$i"; VACUUM; INSERT INTO "$i" SELECT * FROM temp; DROP TABLE temp;"
                sleep 1
            fi
        fi
    done
fi

# Fix the issue in NtpTask and RefMonTask if they exist
if [[ -e $NTPPATH ]] ; then
	cat $NTPPATH | grep -q LogOscillator
	if [[ "$?" -eq "0" ]] ; then
		sed -in 's/LogOscillator/LogNtpStat/' $NTPPATH
	fi
fi

if [[ -e $REFMONPATH ]] ; then
	cat $REFMONPATH | grep -q log_rs_phase_data
	if [[ "$?" -eq "0" ]] ; then
		sed -in 's/log_rs_phase_data/log_ref_mon_statuses/' $REFMONPATH	
	fi
fi

# Unset nullglob and dotglob
shopt -u nullglob
shopt -u dotglob

# Flush all file system buffers
/bin/sync
